/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "adc2.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define ADC2_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void adc2_cfg_setup ( adc2_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    cfg->spi_speed = SPI_SLOW; 
    cfg->spi_mode = SPI_MODE_0;
}

ADC2_RETVAL adc2_init ( adc2_t *ctx, adc2_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return ADC2_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, ADC2_DUMMY );

    spi_master_stop( &ctx->spi );

    return ADC2_OK;

}


void adc2_generic_transfer ( adc2_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_stop( &ctx->spi );
    Delay_1ms( );
    spi_master_start( &ctx->spi );
    Delay_1ms( );
    spi_master_transfer( &ctx->spi, block );
    Delay_1ms( );
    spi_master_stop( &ctx->spi );
    Delay_1ms( );    
}

uint32_t adc2_read_4bytes ( adc2_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t read_reg[ 4 ];
    uint32_t result;

    result = 0x00000000;

    block.tx_buffer = NULL;
    block.rx_buffer = read_reg;
    block.tx_length = 0;
    block.rx_length = 4;

    adc2_generic_transfer ( ctx, &block );

    result = read_reg[ 0 ];
    result <<= 8;
    result |= read_reg[ 1 ];
    result <<= 8;
    result |= read_reg[ 2 ];
    result <<= 8;
    result |= read_reg[ 3 ];
    
    result >>= 7;

    return result;
}

uint32_t adc2_adc_value_read ( adc2_t *ctx )
{
    uint32_t result;
    result = adc2_read_4bytes( ctx );

    while ( result >= 0x01FFFFFF )
    {
        result = adc2_read_4bytes( ctx );
        Delay_10us( );
    }
    result &= 0x003FFFFF;

        if ( result > 0x1FFFFF )
        {
            result = ~result;
            result *= -1;
        }

    return result;
}

uint8_t adc2_check_over_low ( adc2_t *ctx )
{
    uint32_t check;
    uint8_t result;

    check = adc2_read_4bytes( ctx );
    
    check >>= 23;
    result = ( uint8_t ) check ;
    result &= 0x01;

    return result;
}

uint8_t adc2_check_over_high ( adc2_t *ctx )
{
    uint32_t check;
    uint8_t result;

    check = adc2_read_4bytes( ctx );

    check >>= 22;
    result = ( uint8_t ) check ;
    result &= 0x01;

    return result;
}
// ------------------------------------------------------------------------- END

